@@ -12,33 +12,42 @@
 #include <KIO/Global>
 #include <KIO/WorkerBase>
 
+#include <libssh/callbacks.h>
 #include <libssh/libssh.h>
 #include <libssh/sftp.h>
-#include <libssh/callbacks.h>
 
 #include <QQueue>
+#include <QUrl>
 
-namespace KIO {
+#include <QCoroGenerator>
+
+namespace KIO
+{
 class AuthInfo;
-};
+} // namespace KIO
 
 using Result = KIO::WorkerResult;
 
-// sftp_attributes must be freed. Use this ScopedPtr to ensure they always are!
-struct ScopedPointerCustomDeleter
+namespace std
 {
-    static inline void cleanup(sftp_attributes attr)
+template<>
+struct default_delete<struct sftp_attributes_struct> {
+    void operator()(struct sftp_attributes_struct *ptr) const
     {
-        sftp_attributes_free(attr);
+        sftp_attributes_free(ptr);
     }
 };
-typedef QScopedPointer<sftp_attributes_struct, ScopedPointerCustomDeleter> SFTPAttributesPtr;
+} // namespace std
+
+using SFTPAttributesPtr = std::unique_ptr<sftp_attributes_struct>;
 
 class SFTPWorker : public KIO::WorkerBase
 {
 public:
     explicit SFTPWorker(const QByteArray &poolSocket, const QByteArray &appSocket);
     ~SFTPWorker() override;
+    Q_DISABLE_COPY_MOVE(SFTPWorker)
+
     void setHost(const QString &h, quint16 port, const QString &user, const QString &pass) override;
     Q_REQUIRED_RESULT Result get(const QUrl &url) override;
     Q_REQUIRED_RESULT Result listDir(const QUrl &url) override;
@@ -66,14 +75,11 @@ public:
 
     // libssh authentication callback (note that this is called by the
     // global ::auth_callback() call.
-    int auth_callback(const char *prompt, char *buf, size_t len,
-                      int echo, int verify, void *userdata);
+    int auth_callback(const char *prompt, char *buf, size_t len, int echo, int verify, void *userdata);
 
     // libssh logging callback (note that this is called by the
     // global ::log_callback() call.
-    void log_callback(int priority, const char *function, const char *buffer,
-                      void *userdata);
-
+    void log_callback(int priority, const char *function, const char *buffer, void *userdata);
 
     // Must call after construction!
     // Bit rubbish, but we need to return something on init.
@@ -124,7 +130,8 @@ private: // Private variables
      * As SFTP messages are limited to MAX_XFER_BUF_SIZE several requests
      * should be sent simultaneously in order to increase transfer speeds.
      */
-    class GetRequest {
+    class GetRequest
+    {
     public:
         /**
          * Creates a new GetRequest object.
@@ -143,6 +150,7 @@ private: // Private variables
          * in order to avoid memory leaks.
          */
         ~GetRequest();
+        Q_DISABLE_COPY_MOVE(GetRequest)
 
         /**
          * Starts up to maxPendingRequests file requests. Reading is performed in the
@@ -172,6 +180,18 @@ private: // Private variables
         QQueue<Request> m_pendingRequests;
     };
 
+    struct ReadResponse {
+        QByteArray filedata;
+        int error = KJob::NoError;
+    };
+    QCoro::Generator<ReadResponse> asyncRead(sftp_file file, size_t size);
+
+    struct WriteResponse {
+        size_t bytes = 0;
+        int error = KJob::NoError;
+    };
+    QCoro::Generator<WriteResponse> asyncWrite(sftp_file file, QCoro::Generator<ReadResponse> reader);
+
 private: // private methods
     int authenticateKeyboardInteractive(KIO::AuthInfo &info);
 
@@ -188,14 +208,6 @@ private: // private methods
     Q_REQUIRED_RESULT Result sftpGet(const QUrl &url, KIO::fileoffset_t offset = -1, int fd = -1);
     Q_REQUIRED_RESULT Result sftpPut(const QUrl &url, int permissions, KIO::JobFlags flags, int fd = -1);
 
-    /**
-     * sftp_write wrapper breaking buffer into suitable pieces
-     * \param onWritten acts as callback, for each written block.
-     */
-    Q_REQUIRED_RESULT bool sftpWrite(sftp_file fd,
-                                     const QByteArray &buffer,
-                                     const std::function<void(int bytes)> &onWritten);
-
     Q_REQUIRED_RESULT Result sftpCopyGet(const QUrl &url, const QString &src, int permissions, KIO::JobFlags flags);
     Q_REQUIRED_RESULT Result sftpCopyPut(const QUrl &url, const QString &dest, int permissions, KIO::JobFlags flags);
     Q_REQUIRED_RESULT Result sftpSendMimetype(sftp_file file, const QUrl &url);
